﻿using System;
using System.Collections;
using System.Configuration;
using System.Data;
using System.Linq;
using System.Web;
using System.IO;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Xml.Linq;
//Reference to ExcelWriter
using SoftArtisans.OfficeWriter.ExcelWriter;
//Reference to the CSV parser used for retrieving the data sets
using GenericParsing;

namespace CompleteFinancialReport
{
    public partial class Part1 : System.Web.UI.Page
    {
        protected void Page_Load(object sender, EventArgs e)
        {
        }

        protected void RunReport_Click(object sender, EventArgs e)
        {
            try
            {
                //Bind and merge data into a template with ExcelTemplate
                BindDataToTemplate();
            }
            catch (Exception ex)
            {
                ErrorLabel.Visible = true;
                //If OfficeWriter isn't installed, show this error message
                if (ex.Message.Contains("General license key exception"))
                {
                    ErrorLabel.Text = "OfficeWriter does not appear to be" +
                       " installed on this machine. If you have not " +
                       "purchased OfficeWriter, you can download the " +
                       "evaluation version here: " +
                       "http://www.officewriter.com/officewriter-free-evaluation.aspx";
                }
                else
                {
                    ErrorLabel.Text = ex.Message;
                }
            }
        }

        /// <summary>
        /// Retrieve data from CSV files and bind the data to an Excel template
        /// using ExcelTemplate
        /// </summary>
        protected void BindDataToTemplate()
        {
            //Instantiate ExcelTemplate
            ExcelTemplate XLT = new ExcelTemplate();

            //Open the template file
            XLT.Open(Page.MapPath("//templates//Part1_Financial_Template.xlsx"));

            //Create data binding properties for ExcelTemplate
            DataBindingProperties bindingProps = XLT.CreateDataBindingProperties();

            //Create the array of header values. This example only binds a single item
            string[] headerValues = { "2011" };

            //Create the array of header names.
            string[] headerNames = { "FiscalYear" };

            //Get the datatables to be bound into the template.
            //This example uses CSV data, but the data can come from
            //anywhere
            DataTable dtAssets = GetCSVData("//data//Assets.csv");
            DataTable dtLosses = GetCSVData("//data//Losses.csv");
            DataTable dtOther = GetCSVData("//data//Other.csv");

            //Bind the header row data
            XLT.BindRowData(headerValues, headerNames, "Header", bindingProps);

            //Bind the datatables
            XLT.BindData(dtAssets, "Assets", bindingProps);
            XLT.BindData(dtLosses, "Losses", bindingProps);
            XLT.BindData(dtOther, "Other", bindingProps);

            //Process to import the data to the file
            XLT.Process();

            //Save the file
            XLT.Save(Response, "Part1_Output.xlsx", false);
        }


        #region Utility Methods
        /// <summary>
        /// Helper method that uses CSVParser
        /// </summary>
        /// <param name="filename">Filename of CSV</param>
        /// <returns>Populated DataTable</returns>
        protected DataTable GetCSVData(string filename)
        {
            DataTable dt = new DataTable();
            using (GenericParserAdapter parser = new GenericParserAdapter(Page.MapPath(filename)))
            {
                parser.ColumnDelimiter = ',';
                parser.FirstRowHasHeader = true;

                dt = parser.GetDataTable();
            }
            return dt;
        }

        #endregion
    }
}
