﻿using System;
using System.Collections;
using System.Configuration;
using System.Data;
using System.Linq;
using System.IO;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Xml.Linq;
//Reference to ExcelWriter
using SoftArtisans.OfficeWriter.ExcelWriter;
//Reference to the CSV parser used for retrieving the data sets
using GenericParsing;

namespace CompleteFinancialReport
{
    public partial class Part3 : System.Web.UI.Page
    {
        //Declare globals for ExcelTemplate and ExcelApplication objects
        ExcelApplication XLA;
        ExcelTemplate XLT;
        Workbook MERGEDWB;

        protected void RunReport_Click(object sender, EventArgs e)
        {
            try
            {
                //Merge the two worksheets into a single template file
                MergeTemplates();

                //Bind data to the modified template file
                BindDataToMergedTemplate();

            }
            catch (Exception ex)
            {
                ErrorLabel.Visible = true;
                //If OfficeWriter isn't installed, show this error message
                if (ex.Message.Contains("General license key exception"))
                {
                    ErrorLabel.Text = "OfficeWriter does not appear to be" +
                       " installed on this machine. If you have not " +
                       "purchased OfficeWriter, you can download the " +
                       "evaluation version here: " +
                       "http://www.officewriter.com/officewriter-free-evaluation.aspx";
                }
                else
                {
                    ErrorLabel.Text = ex.Message;
                }
            }
        }

        protected void MergeTemplates()
        {
            //Instantiate ExcelApplication
            XLA = new ExcelApplication();

            //Open the first template
            MERGEDWB = XLA.Open(Page.MapPath("//templates//Part1_Financial_Template.xlsx"));

            //Open the second template
            Workbook wbOther = XLA.Open(Page.MapPath("//templates//Part2_Financial_Template.xlsx"));

            //Copy the worksheet from the second template to the merged workbook
            //Give it the same name as the original worksheet
            MERGEDWB.Worksheets.CopySheet(wbOther[0], 1, wbOther[0].Name);

        }

        /// <summary>
        /// Use ExcelTemplate to bind data from CSVs 
        /// </summary>
        protected void BindDataToMergedTemplate()
        {
            XLT = new ExcelTemplate();

            //Open the merged workbook file
            XLT.Open(XLA, MERGEDWB);

            //Create data binding properties for ExcelTemplate
            DataBindingProperties bindingProps = XLT.CreateDataBindingProperties();

            //Create the array of header values. This example only binds a single item
            string[] headerValues = { "2011" };

            //Create the array of header names.
            string[] headerNames = { "FiscalYear" };

            //Get the datatables to be bound into the template.
            //This example uses CSV data, but the data can come from
            //anywhere
            DataTable dtAssets = GetCSVData("//data//Assets.csv");
            DataTable dtLosses = GetCSVData("//data//Losses.csv");
            DataTable dtOther = GetCSVData("//data//Other.csv");

            //Bind the header row data
            XLT.BindRowData(headerValues, headerNames, "Header", bindingProps);

            //Bind the datatables
            XLT.BindData(dtAssets, "Assets", bindingProps);
            XLT.BindData(dtLosses, "Losses", bindingProps);
            XLT.BindData(dtOther, "Other", bindingProps);

            //Process to import the data to the file
            XLT.Process();

            XLT.Save(Page.Response, "Part3_Output.xlsx", false);
        }



        #region Utility Methods
        /// <summary>
        /// Helper method that uses CSVParser
        /// </summary>
        /// <param name="filename">Filename of CSV</param>
        /// <returns>Populated DataTable</returns>
        protected DataTable GetCSVData(string filename)
        {
            DataTable dt = new DataTable();
            using (GenericParserAdapter parser = new GenericParserAdapter(Page.MapPath(filename)))
            {
                parser.ColumnDelimiter = ',';
                parser.FirstRowHasHeader = true;

                dt = parser.GetDataTable();
            }
            return dt;
        }

        #endregion
    }
}
