﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Data.SqlClient;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Xml.Linq;

/*Add this line to use the CSV parsing library for the sample*/
using GenericParsing;

/*Add this line to use ExcelWriter*/
using SoftArtisans.OfficeWriter.ExcelWriter;

namespace ExtendedSalesSummary
{
    public partial class Part2_CSV : System.Web.UI.Page
    {
        /*Declare the global variables that will be used throughout the code.
        * This includes the ExcelApplication object, the ExcelTemplate object,
        * and the Workbook object.*/
        private ExcelApplication xla;
        private ExcelTemplate xlt;
        private Workbook wb;

        //For the list of countries selected by the user
        List<string> selectedCountries;

        protected void Page_Load(object sender, EventArgs e)
        {
        }

        protected void RunReport_Click(object sender, EventArgs e)
        //This function is run when the "Run Report" Button is clicked.
        {
            try
            {
                //Retrieve the user's selections
                selectedCountries = GetListBoxSelections();

                GetReport();
            }
            catch (Exception ex)
            {
                ErrorLabel.Visible = true;
                if (ex.Message.Contains("General license key exception"))
                {
                    ErrorLabel.Text = "OfficeWriter does not appear to be" +
                       " installed on this machine. If you have not " +
                       "purchased OfficeWriter, you can download the " +
                       "evaluation version here: " +
                       "http://www.officewriter.com/officewriter-free-evaluation.aspx";
                }
                else
                {
                    ErrorLabel.Text = ex.Message;
                }

            }
        }

        protected void GetReport()
        {
            //Use ExcelApplication to customize the template based
            //on the countries that were selected by the user
            GenerateTemplate();

            //Use ExcelTemplate to populate the report with datasets
            //from each selected country.
            PopulateTemplate();
        }

        protected void GenerateTemplate()
        {

            //Create a new ExcelApplication and open the existing template 
            //file with ExcelApplication.
            xla = new ExcelApplication();

            wb = xla.Open(Page.MapPath(@"templates\template.xlsx"));

            //For each country, make a copy of the "SimpleTemplate"
            //sheet in the template file and add it to the end of the template
            for (int i = 0; i < selectedCountries.Count; i++)
            {
                wb.Worksheets.CopySheet(wb.Worksheets[0], wb.Worksheets.Count, selectedCountries[i]);
            }

            //Hide the original template
            wb.Worksheets["SimpleTemplate"].Visibility = Worksheet.SheetVisibility.Hidden;

            //Select the first regional sales worksheet in the workbook
            wb.Worksheets[1].Select();

            /*********Part 2*************/
            //Add a worksheet to the template that contains a table of contents
            //and a header image
            AddCoverSheet();
            /**************************/

        }

        protected void AddCoverSheet()
        {
            //Create a new Worksheet called "Summary" at the beginning of the workbook.
            Worksheet ws = wb.Worksheets.CreateWorksheet("Summary", 0);

            //Select this sheet as the active sheet when the workbook opens
            ws.Select();

            //Hide the gridlines in the worksheet
            ws.ShowGridlines = false;

            /*******Inserting the Image********/

            //Make space for the image by setting the row high of row 1
            //Height is in units of 1/72 of an inch
            ws.GetRowProperties(0).Height = 90;

            //Determine which image was selected by the user
            string imagePath = RBImage.SelectedItem.Value;

            //Create an anchor for the picture in cell "A1" with 0 offsets
            Anchor anc = wb.Worksheets[0].CreateAnchor(0, 0, 0, 0);

            //Insert the image into the position specified by the anchor
            Picture logo = ws.Pictures.CreatePicture(Page.MapPath(@imagePath), anc);

            /*******Writing values*************/

            ws.Cells["A3"].Value = "Date Executed :";
            ws.Cells["A4"].Value = "Report For :";
            ws.Cells["A6"].Value = "Table of Contents";

            //Add data markers for the report recipient and date
            ws.Cells["C3"].Value = "%%=WebFormData.Date";
            ws.Cells["C4"].Value = "%%=WebFormData.Name";

            //To add links to other sheets in a workbook, use Excel's native HYPERLINK function
            //For each country, add a HYPERLINK formula under 'Table of Contents'
            //The formula must follow:
            //=HYPERLINK("#DestinationSheet!DesitinationCell", "Display Text")
            for (int i = 2; i < wb.Worksheets.Count; i++)
            {
                string sheetName = wb.Worksheets[i].Name.ToString();
                ws.Cells[7 + i, 1].Formula = "=HYPERLINK(\"#" + sheetName +
                    "!A1\", \"" + sheetName + "\")";
            }

            /******Adding Styles to Text*******/

            //Create a global style for the text labels on the summary sheet
            GlobalStyle labels = wb.CreateStyle();

            //Set the Font to bold and size 12
            labels.Font.Bold = true;
            labels.Font.Size = 12;

            ws.Cells["A3"].ApplyStyle(labels);
            ws.Cells["A4"].ApplyStyle(labels);
            ws.Cells["A6"].ApplyStyle(labels);

            //For 'Table of Contents', set the font to be underlined as well
            ws.Cells["A6"].Style.Font.Underline = Font.UnderlineStyle.Single;

        }

        protected void PopulateTemplate()
        {
            xlt = new ExcelTemplate(); //Create a new ExcelTemplate object

            //Pass the existing ExcelApplication object and open Workbook
            //to the ExcelTemplate object
            xlt.Open(xla, wb);

            //RemoveExtraDataMarkers makes all data markers optional
            //so ExcelWriter will not throw an error if any data markers
            //aren't bound to data
            xlt.RemoveExtraDataMarkers = true;

            //For each country, retrieve and bind data to the worksheet
            DataBindingProperties dataBindProps;

            for (int i = 0; i < selectedCountries.Count; i++)
            {
                string country = selectedCountries[i];

                //Create a new DataBindProperties object
                dataBindProps = xlt.CreateDataBindingProperties();

                //Since the data markers for the copied sheets are all the same, 
                //specify the worksheet that the data should be bound to
                dataBindProps.WorksheetName = country;

                //Retrieve the data and column names for the header on the sheet
                string[] headerValues = { "FY 2008", "Foreign Trade Division", country };
                string[] headerNames = { "FiscalYear", "TradeDivision", "Country" };

                //Bind the header row data to the worksheet
                xlt.BindRowData(headerValues, headerNames, "Header", dataBindProps);

                //Get the data for the top 5 sales and all the sales within the year.
                //In this example, GetCSVData method calls on the GenericParsing library
                //to parse and export data from the CSV files located in the //data directory
                DataTable dts = GetCSVData(Page.MapPath("//data//" + country + "5.csv"));
                DataTable dts2 = GetCSVData(Page.MapPath("//data//" + country + "All.csv"));

                //Call ExcelTemplate.BindData to bind the data to the template
                xlt.BindData(dts, "Top", dataBindProps);
                xlt.BindData(dts2, "Details", dataBindProps);
            }

            /**********Part 2***********/
            //Bind the data from the web form to the cover sheet
            string recipient = TextBox1.Text;

            ////Define the value and names for the cover sheet
            string[] coverValues = { recipient, DateTime.Now.Date.ToString("M/dd/yyyy") };
            string[] coverNames = { "Name", "Date" };

            xlt.BindRowData(coverValues, coverNames, "WebFormData",
                xlt.CreateDataBindingProperties());

            /***************************/

            //Process and stream the final report back to the user
            xlt.Process();
            xlt.Save(Page.Response, "Output.xlsx", false);
        }





        #region Utility methods

        // Gets the items that were selected in the list box
        //Takes the file path for the CSV file and returns
        //a populated DataTable
        DataTable GetCSVData(string csvFileName)
        {
            DataTable dt;
            using (GenericParserAdapter parser = new GenericParserAdapter(csvFileName))
            {
                parser.ColumnDelimiter = ',';
                parser.FirstRowHasHeader = true;

                dt = parser.GetDataTable();
            }
            return dt;
        }

        /// Gets the items that were selected in the list box.
        protected List<string> GetListBoxSelections()
        {
            //Get the ListBox selections and make the appropriate number of copies
            List<string> countryNames = new List<string>();
            foreach (int i in ListBox1.GetSelectedIndices())
            {
                countryNames.Add(ListBox1.Items[i].Text);
            }
            return countryNames;
        }
        #endregion
    }
}