﻿using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using GenericParsing;
/*Add this line to use ExcelWriter*/
using SoftArtisans.OfficeWriter.ExcelWriter;

namespace SimpleExpenseSummary.templates
{
    public partial class Part1 : System.Web.UI.Page
    {
        protected void Page_Load(object sender, EventArgs e)
        {

        }

        protected void RunReport_Click(object sender, EventArgs e)
        {
            try
            {
                //Instantiate the ExcelTemplate object
                ExcelTemplate XLT = new ExcelTemplate();

                //Open the template file
                XLT.Open(Page.MapPath("//templates//part1_template.xlsx"));

                //Create a DataBindingProperties object for binding data to the template
                DataBindingProperties dataProps = XLT.CreateDataBindingProperties();

                //Create a data set for the header values and column names.
                //When working with arrays, the values and column names are placed in separate arrays.

                //This report is for FiscalYear: FY 2004, Division: Canadian Division, Group: Research and Development
                object[] valuesArray = { "FY 2004", "Canadian Division", "Research and Development" };

                //The column names are FiscalYear, Division, Group
                string[] columnNamesArray = { "FiscalYear", "Division", "Group" };

                //Bind the header data row to the data markers in the template (%%=Header.FiscalYear etc.)
                //Use BindRowData to bind a single row of values to a group of data markers
                XLT.BindRowData(valuesArray, columnNamesArray, "Header", dataProps);

                //Get the data for the Top 5 Expenses and All Expenses data sets
                //This data is in CSV files, but the data can come from anywhere
                DataTable dtTop5 = GetCSVData(Page.MapPath("//data//Part1_Top5Expenses.csv"));
                DataTable dtAll = GetCSVData(Page.MapPath("//data//Part1_AllExpenses.csv"));

                //Bind the data set for the Top 5 Expenses and All Expenses to the 
                //data markers in the template (%%=[Top 5 Expenses].Description etc.)
                //Use BindData to import data sources with multiple rows of data
                XLT.BindData(dtTop5, "Top 5 Expenses", dataProps);
                XLT.BindData(dtAll, "All Expenses", dataProps);

                //Process is where the data is imported into the file
                XLT.Process();

                //Stream the output in the response as an attachment
                XLT.Save(Page.Response, "Part1_Output.xlsx", false);

            }
            catch (Exception ex)
            {
                ErrorLabel.Visible = true;
                if (ex.Message.Contains("General license key exception"))
                {
                    ErrorLabel.Text = "OfficeWriter does not appear to be" +
                       " installed on this machine. If you have not " +
                       "purchased OfficeWriter, you can download the " +
                       "evaluation version here: " +
                       "http://www.officewriter.com/officewriter-free-evaluation.aspx";
                }
                else
                {
                    ErrorLabel.Text = ex.Message;
                }

            }
        }

        #region Utility Methods
        //Uses CSV reader
        DataTable GetCSVData(string csvFileName)
        {
            DataTable dt;
            using (GenericParserAdapter parser = new GenericParserAdapter(csvFileName))
            {
                parser.ColumnDelimiter = ',';
                parser.FirstRowHasHeader = true;

                dt = parser.GetDataTable();
            }
            return dt;
        }

        #endregion

    }
}
